==> iota-0.0.8-sources.jar: threads/iota/EntityService.java <==
package threads.iota;

import android.content.Context;
import android.content.SharedPreferences;
import android.util.Log;

import androidx.annotation.NonNull;
import androidx.room.Room;

import java.util.ArrayList;
import java.util.List;

import static androidx.core.util.Preconditions.checkArgument;
import static androidx.core.util.Preconditions.checkNotNull;

public class EntityService {
    private static final String TAG = EntityService.class.getSimpleName();
    private static EntityService SINGLETON = null;
    @NonNull
    private final HashDatabase hashDatabase;

    private static final String PREF_KEY = "prefKey";
    private static final String HOST_KEY = "hostKey";
    private static final String PORT_KEY = "portKey";
    private static final String PROTOCOL_KEY = "protocolKey";
    private static final String TANGLE_TIMEOUT_KEY = "tangleTimeoutKey";

    private static final String TANGLE_PROTOCOL = "https";
    private static final String TANGLE_HOST = "nodes.thetangle.org";
    private static final int TANGLE_PORT = 443;


    public static int getTanglePort(@NonNull Context context) {
        checkNotNull(context);
        SharedPreferences sharedPref = context.getSharedPreferences(PREF_KEY, Context.MODE_PRIVATE);
        return sharedPref.getInt(PORT_KEY, TANGLE_PORT);
    }

    public static void setTanglePort(@NonNull Context context, int timeout) {
        checkNotNull(context);
        checkArgument(timeout > 0);
        SharedPreferences sharedPref = context.getSharedPreferences(PREF_KEY, Context.MODE_PRIVATE);
        SharedPreferences.Editor editor = sharedPref.edit();
        editor.putInt(PORT_KEY, timeout);
        editor.apply();
    }

    public static int getTangleTimeout(@NonNull Context context) {
        checkNotNull(context);
        SharedPreferences sharedPref = context.getSharedPreferences(PREF_KEY, Context.MODE_PRIVATE);
        return sharedPref.getInt(TANGLE_TIMEOUT_KEY, 5);
    }

    public static void setTangleTimeout(@NonNull Context context, int timeout) {
        checkNotNull(context);
        checkArgument(timeout > 0);
        SharedPreferences sharedPref = context.getSharedPreferences(PREF_KEY, Context.MODE_PRIVATE);
        SharedPreferences.Editor editor = sharedPref.edit();
        editor.putInt(TANGLE_TIMEOUT_KEY, timeout);
        editor.apply();
    }

    @NonNull
    public static String getTangleProtocol(@NonNull Context context) {
        checkNotNull(context);
        SharedPreferences sharedPref = context.getSharedPreferences(
                PREF_KEY, Context.MODE_PRIVATE);
        return sharedPref.getString(PROTOCOL_KEY, TANGLE_PROTOCOL);
    }

    public static void setTangleProtocol(@NonNull Context context, @NonNull String token) {
        checkNotNull(context);
        checkNotNull(token);
        SharedPreferences sharedPref = context.getSharedPreferences(
                PREF_KEY, Context.MODE_PRIVATE);
        SharedPreferences.Editor editor = sharedPref.edit();
        editor.putString(PROTOCOL_KEY, token);
        editor.apply();
    }

    @NonNull
    public static String getTangleHost(@NonNull Context context) {
        checkNotNull(context);
        SharedPreferences sharedPref = context.getSharedPreferences(
                PREF_KEY, Context.MODE_PRIVATE);
        return sharedPref.getString(HOST_KEY, TANGLE_HOST);
    }

    public static void setTangleHost(@NonNull Context context, @NonNull String token) {
        checkNotNull(context);
        checkNotNull(token);
        SharedPreferences sharedPref = context.getSharedPreferences(
                PREF_KEY, Context.MODE_PRIVATE);
        SharedPreferences.Editor editor = sharedPref.edit();
        editor.putString(HOST_KEY, token);
        editor.apply();
    }
    private EntityService(@NonNull Context context) {
        checkNotNull(context);

        hashDatabase = Room.databaseBuilder(context,
                HashDatabase.class,
                HashDatabase.class.getSimpleName()).fallbackToDestructiveMigration().build();
    }

    @NonNull
    public static EntityService getInstance(@NonNull Context context) {
        checkNotNull(context);
        if (SINGLETON == null) {
            SINGLETON = new EntityService(context);
        }
        return SINGLETON;
    }

    @NonNull
    public HashDatabase getHashDatabase() {
        return hashDatabase;
    }

    @NonNull
    public Entity insertData(@NonNull Context context,
                             @NonNull String address,
                             @NonNull String data) {
        int timeout = EntityService.getTangleTimeout(context);
        String protocol = EntityService.getTangleProtocol(context);
        int port = EntityService.getTanglePort(context);
        String host = EntityService.getTangleHost(context);

        IOTA iota = new IOTA.Builder().
                protocol(protocol).
                timeout(timeout).
                host(host).
                port(port).build();

        Entity entity = iota.insertTransaction(address, data);
        Hash transactionHash = createHash(entity.getHash());
        insertHash(transactionHash);
        return entity;
    }

    @NonNull
    public Entity insertData(@NonNull Context context,
                             @NonNull String address,
                             byte[] data) {
        int timeout = EntityService.getTangleTimeout(context);
        String protocol = EntityService.getTangleProtocol(context);
        int port = EntityService.getTanglePort(context);
        String host = EntityService.getTangleHost(context);

        IOTA iota = new IOTA.Builder().
                protocol(protocol).
                timeout(timeout).
                host(host).
                port(port).build();

        Entity entity = iota.setContent(address, data);
        Hash transactionHash = createHash(entity.getHash());
        insertHash(transactionHash);
        return entity;
    }


    @NonNull
    public List<Entity> loadEntities(@NonNull Context context, @NonNull String address) {
        checkNotNull(context);
        checkNotNull(address);
        int timeout = EntityService.getTangleTimeout(context);
        String protocol = EntityService.getTangleProtocol(context);
        int port = EntityService.getTanglePort(context);
        String host = EntityService.getTangleHost(context);

        IOTA iota = new IOTA.Builder().
                protocol(protocol).
                timeout(timeout).
                host(host).
                port(port).build();

        return loadEntities(iota, address);
    }


    @NonNull
    private List<Entity> loadEntities(@NonNull IOTA iota, @NonNull String address) {
        checkNotNull(iota);
        checkNotNull(address);

        List<Entity> entities = new ArrayList<>();


        iota.loadAddressTransactions(new IOTA.Filter() {
            @Override
            public boolean acceptHash(@NonNull String hash) {
                return !hasHash(hash);
            }

            @Override
            public void invalidHash(@NonNull String hash) {
                Hash transactionHash = createHash(hash);
                insertHash(transactionHash);
            }

            @Override
            public boolean acceptEntity(@NonNull Entity entity) {
                boolean result = true;
                String hash = entity.getHash();
                if (hasHash(hash)) {
                    result = false;
                } else {
                    Hash transactionHash = createHash(hash);
                    insertHash(transactionHash);
                }
                return result;

            }

            @Override
            public void loadEntity(@NonNull Entity entity) {
                entities.add(entity);
            }

            @Override
            public void invalidEntity(@NonNull Entity entity) {
                String hash = entity.getHash();
                Hash transactionHash = createHash(hash);
                insertHash(transactionHash);
            }

            @Override
            public void error(@NonNull Entity entity, @NonNull Throwable e) {
                String hash = entity.getHash();
                Hash transactionHash = createHash(hash);
                insertHash(transactionHash);
                Log.e(TAG, "" + e.getLocalizedMessage(), e);
            }

        }, address);

        return entities;
    }


    private boolean hasHash(@NonNull String hash) {
        return getHashDatabase().hashDao().hasHash(hash) > 0;
    }


    private Hash createHash(@NonNull String hash) {
        checkNotNull(hash);
        return new Hash(hash, System.currentTimeMillis());
    }

    private void insertHash(@NonNull Hash hash) {
        checkNotNull(hash);
        getHashDatabase().hashDao().insertHash(hash);
    }
}

==> iota-0.1.1-sources.jar: threads/iota/EntityService.java <==
package threads.iota;

import android.content.Context;
import android.content.SharedPreferences;
import android.util.Log;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.room.Room;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.atomic.AtomicReference;

import static androidx.core.util.Preconditions.checkArgument;
import static androidx.core.util.Preconditions.checkNotNull;

public class EntityService {
    private static final String TAG = EntityService.class.getSimpleName();
    private static EntityService SINGLETON = null;
    @NonNull
    private final HashDatabase hashDatabase;

    private static final String PREF_KEY = "prefKey";
    private static final String HOST_KEY = "hostKey";
    private static final String PORT_KEY = "portKey";
    private static final String PROTOCOL_KEY = "protocolKey";
    private static final String TANGLE_TIMEOUT_KEY = "tangleTimeoutKey";

    private static final String TANGLE_PROTOCOL = "https";
    private static final String TANGLE_HOST = "nodes.thetangle.org";
    private static final int TANGLE_PORT = 443;


    public static int getTanglePort(@NonNull Context context) {
        checkNotNull(context);
        SharedPreferences sharedPref = context.getSharedPreferences(PREF_KEY, Context.MODE_PRIVATE);
        return sharedPref.getInt(PORT_KEY, TANGLE_PORT);
    }

    public static void setTanglePort(@NonNull Context context, int timeout) {
        checkNotNull(context);
        checkArgument(timeout > 0);
        SharedPreferences sharedPref = context.getSharedPreferences(PREF_KEY, Context.MODE_PRIVATE);
        SharedPreferences.Editor editor = sharedPref.edit();
        editor.putInt(PORT_KEY, timeout);
        editor.apply();
    }

    public static int getTangleTimeout(@NonNull Context context) {
        checkNotNull(context);
        SharedPreferences sharedPref = context.getSharedPreferences(PREF_KEY, Context.MODE_PRIVATE);
        return sharedPref.getInt(TANGLE_TIMEOUT_KEY, 30);
    }

    public static void setTangleTimeout(@NonNull Context context, int timeout) {
        checkNotNull(context);
        checkArgument(timeout > 0);
        SharedPreferences sharedPref = context.getSharedPreferences(PREF_KEY, Context.MODE_PRIVATE);
        SharedPreferences.Editor editor = sharedPref.edit();
        editor.putInt(TANGLE_TIMEOUT_KEY, timeout);
        editor.apply();
    }

    @NonNull
    public static String getTangleProtocol(@NonNull Context context) {
        checkNotNull(context);
        SharedPreferences sharedPref = context.getSharedPreferences(
                PREF_KEY, Context.MODE_PRIVATE);
        return sharedPref.getString(PROTOCOL_KEY, TANGLE_PROTOCOL);
    }

    public static void setTangleProtocol(@NonNull Context context, @NonNull String token) {
        checkNotNull(context);
        checkNotNull(token);
        SharedPreferences sharedPref = context.getSharedPreferences(
                PREF_KEY, Context.MODE_PRIVATE);
        SharedPreferences.Editor editor = sharedPref.edit();
        editor.putString(PROTOCOL_KEY, token);
        editor.apply();
    }

    @NonNull
    public static String getTangleHost(@NonNull Context context) {
        checkNotNull(context);
        SharedPreferences sharedPref = context.getSharedPreferences(
                PREF_KEY, Context.MODE_PRIVATE);
        return sharedPref.getString(HOST_KEY, TANGLE_HOST);
    }

    public static void setTangleHost(@NonNull Context context, @NonNull String token) {
        checkNotNull(context);
        checkNotNull(token);
        SharedPreferences sharedPref = context.getSharedPreferences(
                PREF_KEY, Context.MODE_PRIVATE);
        SharedPreferences.Editor editor = sharedPref.edit();
        editor.putString(HOST_KEY, token);
        editor.apply();
    }
    private EntityService(@NonNull Context context) {
        checkNotNull(context);

        hashDatabase = Room.databaseBuilder(context,
                HashDatabase.class,
                HashDatabase.class.getSimpleName()).fallbackToDestructiveMigration().build();
    }

    @NonNull
    public static EntityService getInstance(@NonNull Context context) {
        checkNotNull(context);
        if (SINGLETON == null) {
            SINGLETON = new EntityService(context);
        }
        return SINGLETON;
    }

    @NonNull
    public HashDatabase getHashDatabase() {
        return hashDatabase;
    }

    @NonNull
    public Entity insertData(@NonNull Context context,
                             @NonNull String address,
                             @NonNull String data) {
        int timeout = EntityService.getTangleTimeout(context);
        String protocol = EntityService.getTangleProtocol(context);
        int port = EntityService.getTanglePort(context);
        String host = EntityService.getTangleHost(context);

        IOTA iota = new IOTA.Builder().
                protocol(protocol).
                timeout(timeout).
                host(host).
                port(port).build();

        Entity entity = iota.insertTransaction(address, data);
        Hash transactionHash = createHash(entity.getHash());
        insertHash(transactionHash);
        return entity;
    }

    @NonNull
    public Entity insertData(@NonNull Context context,
                             @NonNull String address,
                             byte[] data) {
        int timeout = EntityService.getTangleTimeout(context);
        String protocol = EntityService.getTangleProtocol(context);
        int port = EntityService.getTanglePort(context);
        String host = EntityService.getTangleHost(context);

        IOTA iota = new IOTA.Builder().
                protocol(protocol).
                timeout(timeout).
                host(host).
                port(port).build();

        Entity entity = iota.setContent(address, data);
        Hash transactionHash = createHash(entity.getHash());
        insertHash(transactionHash);
        return entity;
    }

    @Nullable
    public Entity loadEntityByHash(@NonNull Context context, @NonNull String hash) {
        checkNotNull(context);
        checkNotNull(hash);
        int timeout = EntityService.getTangleTimeout(context);
        String protocol = EntityService.getTangleProtocol(context);
        int port = EntityService.getTanglePort(context);
        String host = EntityService.getTangleHost(context);


        IOTA iota = new IOTA.Builder().
                protocol(protocol).
                timeout(timeout).
                host(host).
                port(port).build();


        AtomicReference<Entity> result = new AtomicReference<>(null);

        iota.loadHashTransactions(new IOTA.Filter() {
            @Override
            public boolean acceptHash(@NonNull String hash) {
                return true;
            }

            @Override
            public void invalidHash(@NonNull String hash) {
                Hash transactionHash =
                        createHash(hash);
                insertHash(transactionHash);
            }

            @Override
            public boolean acceptEntity(@NonNull Entity entity) {
                boolean result = true;
                String hash = entity.getHash();
                if (hasHash(hash)) {
                    result = false;
                } else {
                    Hash transactionHash =
                            createHash(hash);
                    insertHash(transactionHash);
                }


                return result;
            }

            @Override
            public void invalidEntity(@NonNull Entity entity) {
                checkNotNull(entity);
                String hash = entity.getHash();
                Hash transactionHash =
                        createHash(hash);
                insertHash(transactionHash);
            }


            @Override
            public void error(@NonNull Entity entity, @NonNull Throwable e) {
                String hash = entity.getHash();
                Hash transactionHash =
                        createHash(hash);
                insertHash(transactionHash);

            }

            @Override
            public void loadEntity(@NonNull Entity entity) {
                result.set(entity);
            }

        }, hash);

        return result.get();
    }


    @NonNull
    public List<Entity> loadEntities(@NonNull Context context, @NonNull String address) {
        checkNotNull(context);
        checkNotNull(address);
        int timeout = EntityService.getTangleTimeout(context);
        String protocol = EntityService.getTangleProtocol(context);
        int port = EntityService.getTanglePort(context);
        String host = EntityService.getTangleHost(context);

        IOTA iota = new IOTA.Builder().
                protocol(protocol).
                timeout(timeout).
                host(host).
                port(port).build();

        return loadEntities(iota, address);
    }


    @NonNull
    private List<Entity> loadEntities(@NonNull IOTA iota, @NonNull String address) {
        checkNotNull(iota);
        checkNotNull(address);

        List<Entity> entities = new ArrayList<>();


        iota.loadAddressTransactions(new IOTA.Filter() {
            @Override
            public boolean acceptHash(@NonNull String hash) {
                return !hasHash(hash);
            }

            @Override
            public void invalidHash(@NonNull String hash) {
                Hash transactionHash = createHash(hash);
                insertHash(transactionHash);
            }

            @Override
            public boolean acceptEntity(@NonNull Entity entity) {
                boolean result = true;
                String hash = entity.getHash();
                if (hasHash(hash)) {
                    result = false;
                } else {
                    Hash transactionHash = createHash(hash);
                    insertHash(transactionHash);
                }
                return result;

            }

            @Override
            public void loadEntity(@NonNull Entity entity) {
                entities.add(entity);
            }

            @Override
            public void invalidEntity(@NonNull Entity entity) {
                String hash = entity.getHash();
                Hash transactionHash = createHash(hash);
                insertHash(transactionHash);
            }

            @Override
            public void error(@NonNull Entity entity, @NonNull Throwable e) {
                String hash = entity.getHash();
                Hash transactionHash = createHash(hash);
                insertHash(transactionHash);
                Log.e(TAG, "" + e.getLocalizedMessage(), e);
            }

        }, address);

        return entities;
    }


    private boolean hasHash(@NonNull String hash) {
        return getHashDatabase().hashDao().hasHash(hash) > 0;
    }


    private Hash createHash(@NonNull String hash) {
        checkNotNull(hash);
        return new Hash(hash, System.currentTimeMillis());
    }

    private void insertHash(@NonNull Hash hash) {
        checkNotNull(hash);
        getHashDatabase().hashDao().insertHash(hash);
    }
}
